/**************************************************************************************************************************************************
*       Copyright  2009-2018 Texas Instruments Incorporated - http://www.ti.com/                                                                 *
***************************************************************************************************************************************************
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*                                                                                                                                                 *
**************************************************************************************************************************************************/


#include "usci_i2c.h"



static uint8_t *PRxData;                     // Pointer to RX data
static uint8_t   RXByteCtr;
static uint8_t *PTxData;                     // Pointer to TX data
static uint8_t  TXByteCtr;
static uint8_t  RPT_Flag;                    // Indicates whether to perform a STOP command at the end of the block (Used to get a RESTART)

static uint8_t  I2C_In_Use_Flag;
static uint8_t  NackFlag;

#define DONT_WAKE_CPU                  0
#define WAKE_CPU                       1

uint8_t Transmit_I2C(uint8_t slaveAddress, uint8_t *i2cArray, uint8_t numBytes, uint8_t rptFlag)
{
    if (I2C_In_Use_Flag == 1)
    {
        return (I2C_FAIL_IN_USE);
    }

    I2C_In_Use_Flag = 1;
    while (UCB0CTLW0 & UCTXSTP);             // Ensure stop condition from previous command has completed

    UCB0I2CSA = slaveAddress;               // Slave Address of the device on the I2C bus
    RPT_Flag = rptFlag;

    PTxData = i2cArray;
    TXByteCtr = numBytes;                   // Load TX byte counter
    __disable_interrupt();                  // Hold interrupts
//    IE2 |= UCB0TXIE;                        // Enable TX interrupt
//    UCB0I2CIE |= UCNACKIE;                  // Enable NACK interrupt

    UCB0IE |= (UCTXIE0 | UCNACKIE);            //Penny Enable NACK and TX interrupt

    UCB0CTLW0  |= (UCTR | UCTXSTT);            // I2C TX, start condition

    while (I2C_In_Use_Flag)                 // Make sure we aren't woken up prematurely
    {
    //    __bis_SR_register(CPUOFF + GIE);    // Enter LPM0 w/ interrupts; First byte received will begin interrupt based processing //Penny
        //Penny
        __bis_SR_register(LPM0_bits | GIE);     // Enter LPM0 w/ interrupts
                                                // Remain in LPM0 until all data is TX'd
    }


    __enable_interrupt();

    if (NackFlag)
    {
        NackFlag = 0;
        return (I2C_FAIL_NACK);
    }

    return (I2C_COMMAND_STARTED);

}



uint8_t Receive_I2C(uint8_t slaveAddress, uint8_t *i2cArray, uint8_t numBytes, uint8_t rptFlag)
{
    if (I2C_In_Use_Flag == 1)
    {
        return (I2C_FAIL_IN_USE);
    }

    I2C_In_Use_Flag = 1;

    while (UCB0CTL1 & UCTXSTP);             // Ensure stop condition (if there was one) from previous command has completed
    //IE2 |= UCB0RXIE;                        // Enable RX
    UCB0IE |= UCRXIE0;                       //Penny enable RX

    UCB0I2CSA = slaveAddress;               // Slave Address of the device on the I2C bus
    RPT_Flag = rptFlag;

    PRxData = i2cArray;                     // Start of RX buffer
    RXByteCtr = numBytes;                   // Load RX byte counter
    UCB0CTLW0 &= ~UCTR;                      // Configure I2C for RX
    __disable_interrupt();                  // Hold interrupts

    if ((numBytes == 1) && (rptFlag == ISSUE_I2C_STOP))
    {
        UCB0CTLW0 |= UCTXSTT;                // I2C Start
        while ((UCB0CTLW0 & UCTXSTT) && (!(UCB0IFG & UCNACKIFG)));         // Poll for START to complete or NACK to occur
        if (UCB0IFG & UCNACKIFG)
        {
            NackFlag = 0;
            I2C_In_Use_Flag = 0;
            __enable_interrupt();
            return (I2C_FAIL_NACK);
        }
        else
            UCB0CTL1 |= UCTXSTP;                // I2C Stop condition - Only one byte received
    }
    else
        UCB0CTL1 |= UCTXSTT;                // I2C RX, start condition

    while (I2C_In_Use_Flag)                 // Make sure we aren't woken up prematurely
    {
        //        __bis_SR_register(CPUOFF + GIE);    // Enter LPM0 w/ interrupts; First byte received will begin interrupt based processing    //Penny
       //Penny
        __bis_SR_register(LPM0_bits | GIE);     // Enter LPM0 w/ interrupts                                                // Remain in LPM0 until all data Rx'd
    }

    __enable_interrupt();

    if (NackFlag)
    {
        NackFlag = 0;
        return (I2C_FAIL_NACK);
    }

    return (I2C_COMMAND_STARTED);

}


uint8_t Write_I2C(uint8_t slaveAddress, uint8_t *i2cArray, uint8_t numBytes)
{
    uint8_t rtn;

    rtn = Transmit_I2C(slaveAddress, i2cArray,numBytes, ISSUE_I2C_STOP);

    while (UCB0CTL1 & UCTXSTP);             // Ensure stop condition completed

    return (rtn);
}



uint8_t Read_I2C(uint8_t slaveAddress, uint8_t commandByte, uint8_t *readBuffer, uint8_t numBytes)
{
    uint8_t rtn;

    rtn = Transmit_I2C(slaveAddress, &commandByte, 1, NO_I2C_STOP_FOR_RESTART);

    if (rtn != I2C_COMMAND_STARTED)
        return (rtn);

    // Assure previous command has completed. This is here in case user application
    while (I2C_In_Use_Flag)                 // Make sure we aren't woken up prematurely

        //__bis_SR_register(CPUOFF + GIE);    // Enter LPM0 w/ interrupts; First byte received will begin interrupt based processing   //Penny

        //Penny
        __bis_SR_register(LPM0_bits | GIE);     // Enter LPM0 w/ interrupts
                                                // Remain in LPM0 until all data
                                                // is TX'd

    //Receive process
    rtn = Receive_I2C(slaveAddress, readBuffer, numBytes, ISSUE_I2C_STOP);

    return (rtn);

}


/*************************************************************************************************************************************************
*  Write_Read_I2C
**************************************************************************************************************************************************/
/*!
* @brief Performs a standard I2C Write of multiple bytes followed by I2C Read Sequence to get data from I2C device
*
* The function sends a dummy write to an I2C Slave Device containing a number of bytes.  An I2C RESTART is issued and then an I2C Read command
* for the indicated number of bytes is executed. The resulting data is placed into the buffer provided by the calling function
*
* param[in]   slaveAddress      I2C Address of the slave device. (7 bit address without R/~W bit)
* param[in]  *writeBuffer       Bytes to write.
* param[in]   numWriteBytes     Number of bytes to write
* param[in]  *readBuffer        Location to place the byte returned from the slave device
* param[in]   numReadBytes      Number of bytes to read
*
* @return  I2C_COMMAND_STARTED, I2C_FAIL_NACK or I2C_FAIL_IN_USE
**************************************************************************************************************************************************/
uint8_t Write_Read_I2C(unsigned int slaveAddress, unsigned char *writeBuffer, unsigned char numWriteBytes,
                             unsigned char *readBuffer, unsigned char numReadBytes)
{
    uint8_t rtn;

    rtn = Transmit_I2C(slaveAddress, writeBuffer, numWriteBytes, NO_I2C_STOP_FOR_RESTART);

    if (rtn != I2C_COMMAND_STARTED)
        return (rtn);

    // Assure previous command has completed. This is here in case user application
    while (I2C_In_Use_Flag)                 // Make sure we aren't woken up prematurely
       // __bis_SR_register(CPUOFF + GIE);    // Enter LPM0 w/ interrupts; First byte received will begin interrupt based processing
        __bis_SR_register(LPM0_bits | GIE);
    //Receive process
    rtn = Receive_I2C(slaveAddress, readBuffer, numReadBytes, ISSUE_I2C_STOP);

    return (rtn);

}






uint16_t counternack = 0;

#pragma vector = USCI_B0_VECTOR
__interrupt void USCI_B0_ISR(void)
{
   switch(__even_in_range(UCB0IV, 0x1e))
   {
       case 0x00: // Vector 0: No interrupts
           break;

       case 0x02: // Vector 2: ALIFG
           break;

       case 0x04: // Vector 4: NACKIFG
           UCB0CTLW0 |= UCTXSTT;                 // resend start if NACK
           break;

       case 0x06: // Vector 6: STTIFG
           break;

       case 0x08: // Vector 8: STPIFG
           break;

       case 0x0a: // Vector 10: RXIFG3
           break;

       case 0x0c: // Vector 12: TXIFG3
           break;

       case 0x0e: // Vector 14: RXIFG2
           break;

       case 0x10: // Vector 16: TXIFG2
           break;

       case 0x12: // Vector 18: RXIFG1
           break;

       case 0x14: // Vector 20: TXIFG1
           break;

       case 0x16: // Vector 22: RXIFG0

           UCB0IFG &= ~UCRXIFG0;                     // Clear USCI_B0 RX int flag
           *PRxData++ = UCB0RXBUF;                 // Move RX Data Buffer to PRxData
           RXByteCtr--;                            // Decrement the counter
           if(RXByteCtr == 1)
           {
               if (RPT_Flag == ISSUE_I2C_STOP)
               {
                   UCB0CTLW0 |= UCTXSTP;            // No Repeated Start: stop condition
               }
           }
           else if (RXByteCtr == 0)
           {
               UCB0IE &= ~UCRXIE0;                   //Disable RX interrupt
               I2C_In_Use_Flag = 0;
           }

           __bic_SR_register_on_exit(LPM0_bits + GIE); // Exit LPM0
           break;

       case 0x18: // Vector 24: TXIFG0
           if(TXByteCtr == 0)
           {
               if(RPT_Flag == ISSUE_I2C_STOP)
               {
                   // Wait ISR and restart processing. Issue a STOP command
                   UCB0CTLW0 |= UCTXSTP;                    // Issue I2C stop condition

               }
               UCB0IE &= ~UCTXIE0;                       // Disable TX interrupt
               UCB0IE &= ~UCNACKIE;                 // Disable NACK interrupt
               UCB0IFG &= ~UCTXIFG0;                     // Clear USCI_B0 TX int flag
               I2C_In_Use_Flag = 0;


           }
           else     // TXByteCtr > 0
           {
               UCB0TXBUF = *PTxData++;                 // Load TX buffer
               TXByteCtr--;                            // Decrement TX byte counter
           }
           __bic_SR_register_on_exit(LPM0_bits + GIE); // Exit LPM0

           break;

       case 0x1a: // Vector 26: BCNTIFG
           break;

       case 0x1c: // Vector 28: clock low time-out
           break;

       case 0x1e: // Vector 30: 9th bit
           break;

       default:
           break;
   }
}





//unsigned char i2c_ISR (void)
//{
//    if ((IFG2 & UCB0RXIFG) && (IE2 & UCB0RXIE))
//    {
//        IFG2 &= ~UCB0RXIFG;                     // Clear USCI_B0 RX int flag
//        *PRxData++ = UCB0RXBUF;                 // Move RX Data Buffer to PRxData
//        RXByteCtr--;                            // Decrement the counter
//        if(RXByteCtr == 1)
//        {
//            if (RPT_Flag == ISSUE_I2C_STOP)
//            {
//                UCB0CTL1 |= UCTXSTP;            // No Repeated Start: stop condition
//            }
//        }
//        else if (RXByteCtr == 0)
//        {
//            IE2 &= ~UCB0RXIE;                   //Disable RX interrupt
//            I2C_In_Use_Flag = 0;
//            return (WAKE_CPU);
//
//        }
//    }
//
//    if ((IFG2 & UCB0TXIFG) && (IE2 & UCB0TXIE))
//    {
//
//        IFG2 &= ~UCB0TXIFG;                     // Clear USCI_B0 TX int flag
//        if(TXByteCtr == 0)
//        {
//            if(RPT_Flag == ISSUE_I2C_STOP)
//            {
//                // Wait ISR and restart processing. Issue a STOP command
//                UCB0CTL1 |= UCTXSTP;                    // Issue I2C stop condition
//
//            }
//            IE2 &= ~UCB0TXIE;                       // Disable TX interrupt
//            UCB0I2CIE &= ~UCNACKIE;                 // Disable NACK interrupt
//            I2C_In_Use_Flag = 0;
//            return (WAKE_CPU);
//
//        }
//        else     // TXByteCtr > 0
//        {
//            UCB0TXBUF = *PTxData++;                 // Load TX buffer
//            TXByteCtr--;                            // Decrement TX byte counter
//        }
//    }
//    return (DONT_WAKE_CPU);
//}


